use ExtUtils::MakeMaker;
use File::Basename;
use blib '..';
use blib '../GdkPixbuf';
use Gtk::Install::Files;
use Gtk::Gdk::Pixbuf::Install::Files;
use Cwd;
require '../Depends.pm';
import ExtUtils::Depends;

require '../tools/gendefs.pl';

mkdir('build', 0777);

$pm = {};

$inc = `gtkimageviewer-config --cflags`;
chomp($inc);
$libs = `gtkimageviewer-config --libs`;
chomp($libs);

@typemaps = @Gtk::Install::Files::typemaps;
$typemaps = [cwd() . '/pkg.typemap', cwd().'/build/GtkImageViewerTypemap'];
@headers = ('build/GtkImageViewerDefs.h');

ExtUtils::Depends::save_config('Gtk::ImageViewer', 'build/IFiles.pm', $inc, $libs, 
	$typemaps, [keys %defs], [@defines], [@headers], $pm);

foreach (@typemaps) {
	print "Add typemap: $Gtk::Install::Files::CORE/$_\n";
	push(@$typemaps, cwd() .'/'.$Gtk::Install::Files::CORE . '/'. $_);
}
foreach (@Gtk::Gdk::Pixbuf::Install::Files::typemaps) {
	print "Add typemap: $Gtk::Gdk::Pixbuf::Install::Files::CORE/$_\n";
	push(@$typemaps, cwd() .'/'.$Gtk::Gdk::Pixbuf::Install::Files::CORE . '/'. $_);
}
push(@defines, @Gtk::Install::Files::defines);
push(@defines, "-I$Gtk::Install::Files::CORE");
push(@defines, @Gtk::Gdk::Pixbuf::Install::Files::defines);
push(@defines, "-I$Gtk::Gdk::Pixbuf::Install::Files::CORE");
push(@defines, "-Ibuild");

$pm->{'GtkImageViewer.pm'} = '$(INST_ARCHLIBDIR)/ImageViewer.pm';
$pm->{'build/GtkImageViewerTypes.pm'} = '$(INST_ARCHLIBDIR)/ImageViewer/Types.pm';

add_c ('build/GtkImageViewerDefs.c');
add_defs('pkg.defs');
add_raw_xs ('xs/GtkImageViewer.xs');
add_headers (qw( <gtk-image-viewer/gtk-image-viewer.h> ));
add_headers (qw( <GdkPixbufDefs.h> ) );

@a = ( '-f', 'GtkImageViewer', 
	'-p', 'Gtk=Gtk',
	'-p', 'GdkGL=Gtk::Gdk::GL',
	'-p', 'GdkRgb=Gtk::Gdk::Rgb',
	'-p', 'Gdk=Gtk::Gdk',
	'-m', 'Gtk::ImageViewer');

%defs = %ExtUtils::Depends::defs;
%headers = %ExtUtils::Depends::headers;
foreach (sort {$defs{$a} <=> $defs{$b}} keys %defs) {
	push @a, '-d', $_;
}
foreach (sort {$headers{$a} <=> $headers{$b}} keys %headers) {
	push @a, '-i', $_;
}

add_raw_xs(gendefs::gendefs(@a));

($xfiles, $object, $ldfrom) = setup_xs();

$depend = { 'build/GtkImageViewerDefs.c' => 'build/GtkImageViewerDefs.h build/PerlGtkImageViewerInt.h' };

foreach (qw( build/GtkImageViewerDefs.h build/PerlGtkImageViewerInt.h)) {
	$pm->{$_} = '$(INST_ARCHLIBDIR)/'. basename($_);
}

write_ext();

@clean = qw(
	build/IFiles.pm build/GtkImageViewerDefs.c build/GtkImageViewerDefs.h 
	build/GtkImageViewerDefs.o build/GtkImageViewerTypemap build/GtkImageViewerTypes.pm 
	build/PerlGtkImageViewerExt.c build/PerlGtkImageViewerExt.h build/PerlGtkImageViewerInt.h 
	build/boxed.xsh build/extension.xsh build/objects.xsh
	build/perl-gtk-image-viewer-ds.pod  build/perl-gtk-image-viewer-ref.pod
);
push(@clean, @ExtUtils::Depends::clean);

# documentation
gendefs::gen_doc('gtk-image-viewer');
system("../tools/gendoc.pl", "-t", 'gtk-image-viewer', keys %ExtUtils::Depends::xs);

foreach $t (@$typemaps) {
print "$t\n", if $t=~ /Pixbuf/i;
}

WriteMakefile(
	'NAME'      => 'Gtk::ImageViewer',
	'VERSION_FROM'	=> 'GtkImageViewer.pm',
	'PM' => $pm,
	'TYPEMAPS' => $typemaps,
	'XS' => $xfiles,
	'XSOPT' => '-noversioncheck',
	'DEFINE'    => join(' ',@defines),
	'dist' => { COMPRESS=>"gzip", SUFFIX=>"gz" },
	'INC' => $inc,
	'LIBS' => [$libs],
	'OBJECT' => $object,
	'clean' => {FILES => join(' ', @clean) },
	#'LDFROM' => $ldfrom,
	'depend' => $depend,
);

sub MY::postamble {

'
Makefile: ' . join(' ',
	sort {$defs{$a} <=> $defs{$b}} keys %defs,
	<*/pkg*.pl>
	) . '

build/PerlGtkImageViewerExt.h build/PerlGtkImageViewerExt.c build/PerlGtkImageViewerInt.h: build/GtkImageViewerDefs.h ../tools/genext.pl
	$(PERL) ../tools/genext.pl GtkImageViewer </dev/null

gendoc:
	../tools/gendoc.pl -t gtk-image-viewer $(XS_FILES)

';



}

sub MY::c_o {
	package MY; # so that "SUPER" works right
	my $inherited = shift->SUPER::c_o(@_);
	$inherited =~ s/CCCMD.*$/$&\n\t\@if test -f `basename \$*.o` -a "`basename \$*.o`" != "\$*.o"; then mv `basename \$*.o` \$*.o; fi/m;
    $inherited;
}

	
